package main

import (
	"fmt"
	"flag"
	"strings"
	"io/ioutil"
	"os"
	"http"
	"bytes"
)

const (
	VERSION_STRING = "Desktop File Generator v.0.1"
	ICON_SEARCH = "https://admin.fedoraproject.org/pkgdb/appicon/show/%s"
)

func createDesktopContents(name string, genericName string, comment string, exec string, icon string, useTerminal bool, categories []string) *bytes.Buffer {
	var buf []byte
	b := bytes.NewBuffer(buf)
	b.WriteString("[Desktop Entry]\n")
	b.WriteString("Encoding=UTF-8\n")
	b.WriteString("Name=" + name + "\n")
	b.WriteString("GenericName=" + genericName + "\n")
	b.WriteString("Comment=" + comment + "\n")
	b.WriteString("Exec=" + exec + "\n")
	b.WriteString("Icon=" + icon + "\n")
	if useTerminal {
		b.WriteString("Terminal=true\n")
	} else {
		b.WriteString("Terminal=false\n")
	}
	b.WriteString("Type=Application\n")
	b.WriteString("Categories=" + strings.Join(categories, ";") + ";\n")
	return b
}

func capitalize(name string) string {
	return strings.ToTitle(name[0:1]) + name[1:];
}

func writeDesktopFile(name string, comment string, categories string) {
	if len(name) < 1 {
		fmt.Println("No name, can't download icon")
		os.Exit(1)
	}
	if len(categories) == 0 {
		categories = "Application"
	}
    buf := createDesktopContents(capitalize(name), capitalize(name), comment, name, name + ".png", false, strings.Split(categories, ";"))
	ioutil.WriteFile(name + ".desktop", buf.Bytes(), 0666)
}

func lineStartsWith(line string, word string) bool {
	return 0 == strings.Index(strings.TrimSpace(line), word)
}

// Return the contents between "" or ''
func betweenQuotes(line string) string {
	for _, quote := range []string{"\"", "'"} {
		if strings.Count(line, quote) == 2 {
			s2 := line[strings.Index(line, quote) + 1:]
			pos2 := strings.Index(s2, quote)
			return s2[:pos2]
		}
	}
	return ""
}

// Does a keyword exist in a lowercase string?
func has(s string, kw string) bool {
  // Replace "-" with " " when searching for keywords
  return -1 != strings.Index(strings.Replace(strings.ToLower(s), "-", " ", -1), " " + kw + " ")
}

func multimediaInDescription(pkgdesc string) bool {
	for _, keyword := range []string{"video", "audio", "sound", "graphics", "draw", "demo"} {
		if has(pkgdesc, keyword) {
			return true
		}
	}
	return false
}

func programmingInDescription(pkgdesc string) bool {
	for _, keyword := range []string{"code", "c", "ide", "programming", "develop", "compile"} {
		if has(pkgdesc, keyword) {
			return true
		}
	}
	return false
}

func gameInDescription(pkgdesc string) bool {
	for _, keyword := range []string{"racing", "game", "arcade", "rts", "mmorpg", "rpg", "fps"} {
		if has(pkgdesc, keyword) {
			return true
		}
	}
	return false
}

// Download icon from the search url in ICON_SEARCH
func writeIconFile(pkgname string) {
	filename := pkgname + ".png"
	var client http.Client
	resp, err := client.Get(fmt.Sprintf(ICON_SEARCH, capitalize(pkgname)))
	if err != nil {
		fmt.Println("Could not download icon")
		os.Exit(1)
	}
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		fmt.Println("Could not dump body")
		os.Exit(1)
	}
	err = ioutil.WriteFile(filename, b, 0666)
	if err != nil {
		fmt.Println("Could not write icon to", filename)
		os.Exit(1)
	}
}

func main() {
	flag.Parse()
	args := flag.Args()
	if len(args) == 0 {
		filedata, err := ioutil.ReadFile("PKGBUILD")
		if err != nil {
			fmt.Fprintf(os.Stderr, "Could not read PKGBUILD\n")
			os.Exit(1)
		}
		filetext := string(filedata)
		pkgdesc := ""
		pkgname := ""
		for _, line := range strings.Split(filetext, "\n") {
			if lineStartsWith(line, "pkgname") {
				pkgname = betweenQuotes(line)
				// If pkgname is not between quotes, get the text after "="
				if (pkgname == "") && (strings.Count(line, "=") == 1) {
					pkgname = strings.TrimSpace(strings.Split(line, "=")[1])
				}
			} else if lineStartsWith(line, "pkgdesc") {
				pkgdesc = betweenQuotes(line)
				// If pkgdesc is not between quotes, get the text after "="
				if (pkgdesc == "") && (strings.Count(line, "=") == 1) {
					pkgdesc = strings.TrimSpace(strings.Split(line, "=")[1])
				}
			}
		}
		categories := ""
		// Try to identify various categories
		if multimediaInDescription(pkgdesc) {
			categories = "Application;Multimedia"
		} else if programmingInDescription(pkgdesc) {
			categories = "Application;Development"
		} else if gameInDescription(pkgdesc) {
			categories = "Application;Game"
		}
		// 
		writeDesktopFile(pkgname, pkgdesc, categories)
		writeIconFile(pkgname)
	} else {
		fmt.Println(VERSION_STRING)
	}
}
